/*
 * CtrlUtils is a JavaScript class with a set of functions for handling all 
 * the different attribute controls. In Change 5.0 the controls were streamlined
 * and their code was added to external JavaScript files as a way of making
 * the HTML pages (show, transition, etc) load faster. The more JavaScript a page
 * has the longer is takes to load. 
 */
function CtrlUtils()
{
}


CtrlUtils.isModified = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	
	if (CtrlUtils._isLegacyAttrControl(ctrl))
	{
		var legacyFunction = window[attributeId + "_is_Modified"];
		return legacyFunction ?
			legacyFunction() :
			window.undefined;
	}
	
	var override = ctrl.getAttribute("onIsModified");
	
	if (override != null)
	{
		override = CtrlUtils._determineOverride(override);
		return override(attributeId);
	}
		
	var isModified = ctrl.getAttribute("isModified");
	return CtrlUtils.equalsTrue(isModified);
};


CtrlUtils.isModifiedByUser = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	
	if (CtrlUtils._isLegacyAttrControl(ctrl))
	{
		var legacyFunction = window[attributeId + "_is_Modified_by_User"];
		return legacyFunction ?
			legacyFunction() :
			window.undefined;
	}
	
	var override = ctrl.getAttribute("onIsModifiedByUser");
	
	if (override != null)
	{
		override = CtrlUtils._determineOverride(override);
		return override(attributeId);
	}
		
	var isModifiedByUser = ctrl.getAttribute("isModifiedByUser");
	return CtrlUtils.equalsTrue(isModifiedByUser);
};


CtrlUtils.isESignatureModified = function(attributeId)
{
	if (CtrlUtils.isReadOnly(attributeId))
	{
		return false;
	}
		
	return CtrlUtils.hasValue(attributeId);
};


CtrlUtils.isRequired = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	var isRequired = ctrl.getAttribute("isRequired");
	return CtrlUtils.equalsTrue(isRequired);
};


CtrlUtils.isExternal = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	var isExternal = ctrl.getAttribute("isExternal");
	return CtrlUtils.equalsTrue(isExternal);
};


CtrlUtils.needsDepCheck = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	var needsDepCheck = ctrl.getAttribute("needsDepCheck");
	return CtrlUtils.equalsTrue(needsDepCheck);
};


CtrlUtils.formType = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	
	if (CtrlUtils._isLegacyAttrControl(ctrl))
	{
		var legacyFunction = window[attributeId + "_formtype"];
		return legacyFunction ?
			legacyFunction :
			window.undefined;
	}
	
	return ctrl.getAttribute("formType");
};


CtrlUtils.webType = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	return ctrl.getAttribute("webType");
};


CtrlUtils.needsUpdate = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	var needsUpdate = ctrl.getAttribute("needsUpdate");
	return CtrlUtils.equalsTrue(needsUpdate);
};


CtrlUtils.isReadOnly = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	
	if (CtrlUtils._isLegacyAttrControl(ctrl))
	{
		var legacyFunction = window[attributeId + "_is_ReadOnly"];
		return legacyFunction ?
			legacyFunction() :
			window.undefined;
	}
	
	var isReadOnly = ctrl.getAttribute("isReadOnly");
	return CtrlUtils.equalsTrue(isReadOnly);
};


CtrlUtils.valueListboxChangeFocus = function (attributeId, markDirty)
{
	if (IsFormLoadingSilent())
		return;
		
	var ctrl = document.getElementById(attributeId);
	
	if (CtrlUtils._isLegacyAttrControl(ctrl))
	{
		var legacyFunction;
		if (markDirty)
			legacyFunction = window[attributeId + "_ChangeFocusNotDirty"];
		else
			legacyFunction = window[attributeId + "_ChangeFocus"];
			
		return legacyFunction ?
			legacyFunction() :
			window.undefined;
	}
	
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return;

	if (ctrl.nodeName != "SELECT")
		return;
		
	ctrl.setAttribute("currentValue", ctrl.options[ctrl.selectedIndex].value);
	
	if (ctrl.options[0].selected)
	{
		ctrl.setAttribute("currentValue", '');
	}

	if (markDirty)
	{
		CtrlUtils._markAsChanged(ctrl);
	}
};


CtrlUtils.userChangeFocus = function (attributeId, markDirty)
{
	if (IsFormLoadingSilent())
		return;
		
	var ctrl = document.getElementById(attributeId);
	
	if (CtrlUtils._isLegacyAttrControl(ctrl))
	{
		var legacyFunction;
		if (markDirty)
			legacyFunction = window[attributeId + "_ChangeFocusNotDirty"];
		else
			legacyFunction = window[attributeId + "_ChangeFocus"];
			
		return legacyFunction ?
			legacyFunction() :
			window.undefined;
	}
	
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return;

	if (ctrl.nodeName != "SELECT")
		return;
		
	if (ctrl.options[ctrl.selectedIndex].text == '')
	{
		ctrl.options[0].selected = true;
	}

	ctrl.setAttribute("currentValue", ctrl.options[ctrl.selectedIndex].value);
	
	if (ctrl.options[0].selected)
	{
		ctrl.setAttribute("currentValue", '');
	}

	if (markDirty)
	{
		CtrlUtils._markAsChanged(ctrl);
	}
};


CtrlUtils.listboxChangeFocus = function (attributeId, markDirty)
{
	if (IsFormLoadingSilent())
		return;
		
	var ctrl = document.getElementById(attributeId);
	
	if (CtrlUtils._isLegacyAttrControl(ctrl))
	{
		var legacyFunction;
		if (markDirty)
			legacyFunction = window[attributeId + "_ChangeFocusNotDirty"];
		else
			legacyFunction = window[attributeId + "_ChangeFocus"];
			
		return legacyFunction ?
			legacyFunction() :
			window.undefined;
	}
	
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return;

	if (ctrl.nodeName != "SELECT")
		return;
		
	if (ctrl.options[ctrl.selectedIndex].text == '')
	{
		ctrl.options[0].selected = true;
	}

	ctrl.setAttribute("currentValue", ctrl.options[ctrl.selectedIndex].text);
	
	if (ctrl.options[0].selected)
	{
		ctrl.setAttribute("currentValue", '');
	}
	
	CtrlUtils._markAsChanged(ctrl);	
	ctrl.depFunction();
};


CtrlUtils.onSelectMouseOver = function(attributeId, ancestors, fromFocusEvent)
{
	var ctrl = document.getElementById(attributeId);
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return;
		
	if (ctrl.nodeName != "SELECT")
		return;
		
	if ((!CtrlUtils.isExternal(attributeId)) || (!CtrlUtils.needsUpdate(attributeId)))
	{
		return;
	}
	
	ctrl.disabled = true;
	
	function callback(listboxValues)
	{
		// Get the controls current value.
		var currentValue = ctrl.getAttribute("currentValue");
		// Load the list box with the new values from the script.
		CtrlUtils.loadListbox(attributeId, listboxValues, false);
		// Instead of using the built in setValue functions do this manually to avoid loading/modification/save issues.
		var index = CtrlUtils._getListboxIndexOf(ctrl, currentValue);
		
		if (index != -1)
		{
			ctrl.selectedIndex = index;
		}
		else if (currentValue != '')
		{
			ctrl.options[ctrl.length] = new Option(currentValue, -1);
			ctrl.options.selectedIndex = ctrl.length - 1;
		}
		
		ctrl.disabled = false;

		if (fromFocusEvent)
			ctrl.focus();
	}	
	
	dwr.engine.setErrorHandler(function(message)		
	{
		ctrl.disabled = false;
		
		var ex = {name:'java.lang.Throwable',  message:'Error'};		
		dwr.engine.defaultErrorHandler(message, ex);
	});
	
	
	ListboxUtilsAjax.getListboxValues(
		token,
		ctrl.getAttribute("literalName"), 
		ctrl.getAttribute("formType"), 
		ancestors, 
		callback);
};


CtrlUtils.checkboxOnClick = function(ctrl)
{
	if (IsFormLoadingSilent())
		return;
		
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return;
		
	ctrl.setAttribute("currentValue", (ctrl.checked ? 'true':'false'));
	CtrlUtils._markAsChanged(ctrl);
};


CtrlUtils.stringOnChangeText = function(ctrl)
{
	if (IsFormLoadingSilent())
		return;

	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return;
	
	ctrl.setAttribute("currentValue", ctrl.value);
	CtrlUtils._markTextAsChanged(ctrl);
};


CtrlUtils.dateOnChangeText = function(ctrl)
{
	var isReadOnly = ctrl.getAttribute("isReadOnly");
	
	if (IsFormLoadingSilent() || CtrlUtils.equalsTrue(isReadOnly) || (ctrl.getAttribute("currentValue") == ctrl.value))
	{
		return;
	}

	var oriCtrlValue = ctrl.getAttribute("currentValue");
	var formType = ctrl.getAttribute("formType");
	var tempValue;
	
	switch (formType)
	{
		case "QUERY":
		case "SUBMIT":
		case "PREF":
			// If your date format does not require a time, use this call instead...
			//ctrl.value = Valid_Date_No_Time(ctrl.getAttribute("currentValue"), formType);
			tempValue = Valid_Date_With_Time(ctrl.value, formType);
			break;
		case "SHOW":
		case "TRANSITION":
			// If your date format does not require a time, use this call instead...
			//ctrl.value = Valid_Date_No_Time(ctrl.getAttribute("currentValue"), "SUBMIT");
			tempValue = Valid_Date_With_Time(ctrl.value, "SUBMIT");
			break;
		default:
			tempValue = Valid_Date(ctrl.value);
			break;
	}

	if ((tempValue.length > 0 && ctrl.value.length > 0) 
		|| ctrl.value.length == 0)
	{
		ctrl.value = tempValue;
		ctrl.setAttribute("currentValue", tempValue);
		CtrlUtils._markAsChanged(ctrl);
	}
	else
	{
		ctrl.value = oriCtrlValue;
		ctrl.select();
	}
};


//Requires that problem_number is defined on the form.
CtrlUtils.duplicateChangeText = function(attributeId, dcm_context_dbid, dcm_dbid, dcm_delimeter, errorMessage)
{
	if (IsFormLoadingSilent())
		return;
	
	var ctrl = document.getElementById(attributeId);
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return;
		
	ctrl.value = dojo.trim(ctrl.value);
	if(ctrl.getAttribute("formType") == 'TRANSITION' && ctrl.value.length > 0)
	{
		if(ctrl.value.toLowerCase() == CtrlUtils.getValue('problem_number').toLowerCase())
		{
			alert(errorMessage);
			ctrl.value	= '';
			ctrl.focus()
			return;
		}

		if(CtrlUtils.getValue('problem_number').indexOf(dcm_dbid + dcm_delimeter) != -1)
		{
			if(CtrlUtils.getValue('problem_number').substring(dcm_dbid.length + dcm_delimeter.length) == ctrl.value)
			{
				alert(errorMessage);
				ctrl.value	= '';
				ctrl.focus()
				return;
			}
		}
	}

	ctrl.setAttribute("currentValue", ctrl.value);
	CtrlUtils._markTextAsChanged(ctrl);
}


CtrlUtils.numberOnChangeText = function(ctrl, errorMessage)
{
	if (IsFormLoadingSilent())
		return;
			
	if (!CtrlUtils._isValidNumber(ctrl.value, errorMessage))
	{
		ctrl.value = "";
		return;
	}

	ctrl.setAttribute("currentValue", ctrl.value);
	CtrlUtils._markAsChanged(ctrl);
};


CtrlUtils.loadListbox = function(attributeId, list, isParentDirty)
{
	var ctrl = document.getElementById(attributeId);
	
	if (CtrlUtils._isLegacyAttrControl(ctrl))
	{
		var legacyFunction = window[attributeId + "_LoadListBox"];
		
		return legacyFunction ?
			legacyFunction(ctrl, list, isParentDirty) :
			window.undefined;
	}
	
	var override = ctrl.getAttribute("onLoadListbox");
	
	if (override != null)
	{
		override = CtrlUtils._determineOverride(override);
		return override(attributeId, list, isParentDirty);
	}
		
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return;

	if (ctrl.nodeName != "SELECT")
		return;

	if (list == null)
	{
		ctrl.length = 0;
		return;
	}
			
	ctrl.setAttribute("needsUpdate", "false");
	ctrl.length = 0;

	for (var i = 0; i < list.length; i++)
	{
		ctrl.options[ctrl.length] = new Option(list[i], i);
	}

	if (ctrl.options.length > 0)
		ctrl.options[0].selected = true;
};


CtrlUtils.setValue = function(attributeId, value, isModified)
{
	var ctrl = document.getElementById(attributeId);
	
	if (CtrlUtils._isLegacyAttrControl(ctrl))
	{
		var legacyFunction = window[attributeId + "_Set_Value"];
		
		return legacyFunction ?
			legacyFunction(value, isModified) :
			window.undefined;
	}
	
	var override = ctrl.getAttribute("onSetValue");
	
	if (override != null)
	{
		override = CtrlUtils._determineOverride(override);
		return override(attributeId, value, isModified);
	}
	
	//Do nothing for types without overrides.
};


CtrlUtils.setListboxValue = function(attributeId, value, isModified)
{
	var ctrl = document.getElementById(attributeId);
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return;
		
	ctrl.setAttribute("isModified", isModified);
	CtrlUtils._setListboxValue(ctrl, value);
};


CtrlUtils.setValueListboxValue = function(attributeId, value, isModified)
{
	var ctrl = document.getElementById(attributeId);
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return;
		
	ctrl.setAttribute("isModified", isModified);
	CtrlUtils._setValueListboxValue(ctrl, value);
};


CtrlUtils.setReadOnlyAndLogValue = function(attributeId, value, isModified)
{
	var ctrl = document.getElementById(attributeId);
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return;
		
	ctrl.setAttribute("isModified", isModified);
	ctrl.setAttribute("currentValue", value);
};


CtrlUtils.setTextValue = function(attributeId, value, isModified)
{
	var ctrl = document.getElementById(attributeId);
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return;
		
	ctrl.setAttribute("isModified", isModified);
	ctrl.value = value;
	ctrl.setAttribute("currentValue", value);
};


CtrlUtils.setToggleValue = function(attributeId, value, isModified)
{
	var ctrl = document.getElementById(attributeId);
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return;
		
	ctrl.setAttribute("isModified", isModified);
	ctrl.checked = value;
	ctrl.setAttribute("currentValue",(value ? 'true':'false'));
};


CtrlUtils.getListboxPosition = function (attributeId, compareValue)
{
	var ctrl = document.getElementById(attributeId);
	
	if (CtrlUtils._isLegacyAttrControl(ctrl))
	{
		var legacyFunction = window[attributeId + "_ListBoxPosition"];
		
		return legacyFunction ?
			legacyFunction(compareValue) :
			window.undefined;
	}
	
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return 0;

	var reloadList = CtrlUtils.getReloadList(attributeId);

	if (reloadList == null)
		return 0;

	if (reloadList.length == 0)
		return ctrl.selectedIndex;

	for (var i = 0; i < reloadList.length; i++)
	{
		if (reloadList[i] == compareValue)
		{
			return i;
		}
	}

	return 0;
};


CtrlUtils.setFocus = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	
	if (CtrlUtils._isLegacyAttrControl(ctrl))
	{
		var legacyFunction = window[attributeId + "_focus"];
		return legacyFunction ?
			legacyFunction() :
			window.undefined;
	}
	
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (!CtrlUtils.equalsTrue(isReadOnly))
	{
		ctrl.focus();
	}
};


CtrlUtils.hasValue = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	
	if (CtrlUtils._isLegacyAttrControl(ctrl))
	{
		var legacyFunction = window[attributeId + "_has_Value"];
		
		return legacyFunction ?
			legacyFunction() :
			window.undefined;
	}
	
	var override = ctrl.getAttribute("onHasValue");
	
	if (override != null)
	{
		override = CtrlUtils._determineOverride(override);
		return override(attributeId);
	}
	
	//No over ride means no value.
	return false;
};


CtrlUtils.esigHasValue = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return true;
	
	return ((ctrl.getAttribute("currentValue") != '') && (ctrl.getAttribute("currentValue") != 'NO_ACTION') && (ctrl.getAttribute("currentValue").length > 0));
};


CtrlUtils.valueExists  = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return true;

	if ((ctrl.getAttribute("currentValue") != '') && (ctrl.getAttribute("currentValue").length > 0))
	{
		return true;
	}
		
	return false;		
}


CtrlUtils.textHasValue = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return true;
		
	var temp = ctrl.getAttribute("currentValue");
		
	if (temp == null)
		return false;
	
	temp = temp.replace(/^\s+/, '');
	temp = temp.replace(/\s+$/, '');	
	return ((temp != '') && (temp.length > 0));
};

CtrlUtils.getListboxCtrl = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	
	if (CtrlUtils._isLegacyAttrControl(ctrl))
	{
		var legacyFunction = window[attributeId + "_ListboxCtrl"];
		return legacyFunction ?
			legacyFunction() :
			window.undefined;
	}
	
	return ctrl;
};

CtrlUtils.selectHasValue = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return true;
		
	return ctrl.selectedIndex > 0;
};


CtrlUtils.hasValueTrue = function(attributeId)
{
	return true;
}


CtrlUtils.getValue = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	
	if (CtrlUtils._isLegacyAttrControl(ctrl))
	{
		var legacyFunction = window[attributeId + "_Value"];
		
		return legacyFunction ?
			legacyFunction() :
			window.undefined;
	}
	
	var override = ctrl.getAttribute("onGetValue");
	
	if (override != null)
	{
		override = CtrlUtils._determineOverride(override);
		return override(attributeId);
	}
		
	return ctrl.getAttribute("currentValue");
};


CtrlUtils.getESigValue = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	var isReadOnly = ctrl.getAttribute("isReadOnly");
	
	if (CtrlUtils.equalsTrue(isReadOnly))
	{
		return "NO_ACTION";
	}
	
	return ctrl.getAttribute("currentValue");
};


CtrlUtils.getListboxValue = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	var isReadOnly = ctrl.getAttribute("isReadOnly");
	
	if (CtrlUtils.equalsTrue(isReadOnly))
	{
		return ctrl.getAttribute("currentValue");
	}
	
	return ctrl.options[0].selected ? '' : ctrl.getAttribute("currentValue");
};


// NOTE: Must include calendar.js on template to use this function
CtrlUtils.getQueryValue = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	
	if (CtrlUtils._isLegacyAttrControl(ctrl))
	{
		var legacyFunction = window[attributeId + "_QueryValue"];
		
		return legacyFunction ?
			legacyFunction() :
			window.undefined;
	}
	
	return QueryValue(ctrl.getAttribute("currentValue"), attributeId);
};


CtrlUtils.setListboxSelected = function(attributeId, index, isModified)
{
	var ctrl = document.getElementById(attributeId);
	
	if (CtrlUtils._isLegacyAttrControl(ctrl))
	{
		var legacyFunction = window[attributeId + "_Set_Selected"];
		return legacyFunction ?
			legacyFunction(index, isModified) :
			window.undefined;
	}
	
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return;

	if (ctrl.options.length > 0)
	{
		if (isNaN(index) || (index < 0))
			index = 0;

		ctrl.setAttribute("isModified", isModified);
		ctrl.options[index].selected = true;
		ctrl.setAttribute("currentValue", ctrl.options[index].text);
		ctrl.depFunction();
	}
};

CtrlUtils.getDepList = function(attributeId, pos)
{
	var ctrl = document.getElementById(attributeId);
	
	if (CtrlUtils._isLegacyAttrControl(ctrl))
	{
		var legacyFunction = window[attributeId + "_GetDepList"];
		
		return legacyFunction ?
			legacyFunction(pos) :
			window.undefined;
	}
	
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return null;
		
	ctrl.depListLoadFcn(pos);
	
	return ctrl.depLists;
};


CtrlUtils.getReloadList = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	
	if (CtrlUtils._isLegacyAttrControl(ctrl))
	{
		var legacyFunction = window[attributeId + "_GetReloadList"];
		return legacyFunction ?
			legacyFunction() :
			window.undefined;
	}
	
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return null;
	
	if (ctrl.reload.length == 0)
		ctrl.reloadFcn();
	
	return ctrl.reload;
};


CtrlUtils.autoSizeOnClick = function(button, attributeId, rows, originalLabel, originalTitle, restoreLabel, restoreTitle)
{
	var ctrl = document.getElementById(attributeId);
	
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return;
		
	if (button.value != originalLabel) // restore the textbox size
	{
		ctrl.rows = rows;
		button.value = originalLabel;
		button.title = originalTitle;
	}
	else // autosize the textbox
	{
		var lines = ctrl.getAttribute("currentValue").split('\n');
		var rows = 0;

		for (var i = 0; i < lines.length; i++)
		{
			if (lines[i].length > ctrl.cols)
			{
				rows += Math.ceil((lines[i].length / ctrl.cols) * 1.1);
			}
			else
			{
				rows += 1;
			}
		}

		ctrl.rows = rows;
		button.value = restoreLabel;
		button.title = restoreTitle;
	}
};


CtrlUtils.growOnClick = function(attributeId, adjustmentFactor)
{
	var ctrl = document.getElementById(attributeId);
	
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return;
		
	ctrl.rows += adjustmentFactor;
};


CtrlUtils.shrinkOnClick = function(attributeId, rows, adjustmentFactor)
{
	var ctrl = document.getElementById(attributeId);
	
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return;
	
	if ((ctrl.rows - adjustmentFactor) >= rows)
	{
		ctrl.rows -= adjustmentFactor;
	}
	else
	{
		ctrl.rows = rows;
	}
};


CtrlUtils.buttonOnclick = function(ctrl)
{
	if (IsFormLoading())
		return;
		
	// Add code here to process button push
	ctrl.setAttribute("isModified", 'true');
};


CtrlUtils.equalsTrue = function(value)
{
	if ((value == true) || (value == "true"))
	{
		return true;
	}

	return false;
};


CtrlUtils.setOrigValue = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	
	if (CtrlUtils._isLegacyAttrControl(ctrl))
	{
		window[attributeId + "_orig_value"] = window[attributeId];
		return;
	}
	
	var currentValue = ctrl.getAttribute("currentValue");
	ctrl.setAttribute("origValue",currentValue);
};


CtrlUtils.removeInvalidListboxEntry = function(attributeId, option)
{
	var i;
	var ctrl = document.getElementById(attributeId);

	for (i = 0; i < ctrl.options.length; i++)
	{
		if (ctrl.options[i].text == option)
		{
			ctrl.remove(i);
			return;
		}
	}
};


CtrlUtils.setUserList = function(attributeId, list)
{
	var ctrl = document.getElementById(attributeId);
	
	if (CtrlUtils._isLegacyAttrControl(ctrl))
	{
		var legacyFunction = window[attributeId + "_setListboxValue"];
		
		return legacyFunction ?
			legacyFunction(list) :
			window.undefined;
	}
	
	var isReadOnly = ctrl.getAttribute("isReadOnly");
	
	if (CtrlUtils.equalsTrue(isReadOnly))
		return;
	
	if (ctrl.nodeName != "SELECT")
		return;
		
	if (list == null)
	{
		ctrl.length = 0;
		return;
	}
	
	var currentLabel = ctrl.options[ctrl.selectedIndex].text;
	ctrl.length = 0;
	
	var foundValue = false;
	var count = list.length;
	
	for (var i = 0; i < count; i++)
	{
		ctrl.options[ctrl.length] = new Option(list[i].displayName, list[i].uid);
		
		if (list[i].uid == ctrl.getAttribute("currentValue"))
		{
			foundValue = true;
			ctrl.options[i].selected = true;
		}
	}
	
	if ((!foundValue) && (ctrl.getAttribute("currentValue") != ''))
	{
		ctrl.options[ctrl.length] = new Option(currentLabel, ctrl.getAttribute("currentValue"));
		ctrl.options[i].selected = true;
	}
	else
	{
		if ((ctrl.options.length > 0) && (ctrl.options.selectedIndex < 0))
		{
			ctrl.options[0].selected = true;
		}
	}
};


/**
 * Set the dependent external list box value to 'Any' 
 */
CtrlUtils.clearExternalDepdentListbox = function(attributeId)
{
	var ctrl = document.getElementById(attributeId);
	
	ctrl.length = 1;
	ctrl.setAttribute("needsUpdate", "true");
		
	ctrl.options[ctrl.length] = new Option("", -1);
	ctrl.options.selectedIndex = ctrl.length - 1;
	
	CtrlUtils.listboxChangeFocus(attributeId);
}

CtrlUtils.createExtendedTextEditor = function(attributeId, focusOnLoad)
{
	var originalDiv = dojo.byId(attributeId);
	
	if ((CtrlUtils.formType(attributeId) != "SUBMIT") && (attributeId != "_COMMENTS"))
	{
		dojo.byId(attributeId + "_EDIT_BUTTON").style.display = "none";
		dojo.byId(attributeId + "_CANCEL_BUTTON").style.display = "";
	}
	
	var height = originalDiv.getAttribute("height");
	originalDiv.style.height = "";
	
	var editor = new dijit.Editor(
		{
			height: height + "px",	
			id: attributeId, 
			focusOnLoad: focusOnLoad,
			plugins: ['formatBlock','fontSize','|','bold','italic','underline','strikethrough','|','justifyLeft','justifyCenter','justifyRight','|','insertOrderedList','insertUnorderedList','|','indent','outdent','|','foreColor','hiliteColor','|','createLink','insertImage','|','removeFormat']
		}, 
		originalDiv
	);
	
	dojo.byId(attributeId + "_resizing").style.display = "";
	
	//Dojo transforms the DIV, now we need to re-apply all our attributes.
	var element = dojo.byId(attributeId);
	
	// Since the editor is up we know they might modify it, have to have special on modify overrides to know for sure.
	element.setAttribute("isModified", true);
	element.setAttribute("isModifiedByUser", true);
	
	//Have to be careful and copy everything that is currently in use.
	element.setAttribute("onIsModified", originalDiv.getAttribute("onIsModified"));
	element.setAttribute("onIsModifiedByUser", originalDiv.getAttribute("onIsModifiedByUser")); 
	element.setAttribute("formType", CtrlUtils.formType(attributeId));
	element.setAttribute("hasSubmitContent", originalDiv.getAttribute("hasSubmitContent"));
	element.setAttribute("isReadOnly", CtrlUtils.isReadOnly(attributeId));
	element.setAttribute("webType", CtrlUtils.webType(attributeId));
	element.setAttribute("onSetValue", originalDiv.getAttribute("onSetValue"));
	element.setAttribute("onHasValue", originalDiv.getAttribute("onHasValue"));
	element.setAttribute("onGetValue", originalDiv.getAttribute("onGetValue"));
	element.setAttribute("version", originalDiv.getAttribute("version"));
	element.setAttribute("currentValue", originalDiv.getAttribute("currentValue"));
	element.setAttribute("origValue", originalDiv.getAttribute("origValue"));
	element.setAttribute("editMode", true);
	
	if (focusOnLoad)
	{
		var timeoutfunc = function() {editor.placeCursorAtEnd();};
		window.setTimeout(timeoutfunc, 0);
	}
};


CtrlUtils.cancelExtendedTextEditor = function(attributeId)
{
	var editor = dijit.byId(attributeId);
	editor.close(false);
	editor.destroyRecursive(false);
	
	var div = dojo.byId(attributeId);
	div.innerHTML = div.getAttribute("origValue");
	
	// The user canceled the edit operation, this means they didn't modify it.
	div.setAttribute("isModified", false);
	div.setAttribute("isModifiedByUser", false);
	div.setAttribute("editMode", false);
			
	dojo.byId(attributeId + "_EDIT_BUTTON").style.display = "";
	dojo.byId(attributeId + "_CANCEL_BUTTON").style.display = "none";
	dojo.byId(attributeId + "_resizing").style.display = "none";
};


CtrlUtils.extendedTextIsModified = function(attributeId)
{
	var ctrl = dojo.byId(attributeId);
	var isEditMode = ctrl.getAttribute("editMode");
	var origValue = ctrl.getAttribute("origValue");
	var currentValue;
	
	if (CtrlUtils.equalsTrue(isEditMode))
	{
		currentValue = CtrlUtils.getExtendedTextValue(attributeId);
	}
	else
	{
		currentValue = ctrl.getAttribute("currentValue");
	}
	
	var changed = currentValue != origValue;
		
	if ((currentValue == "") && (!origValue))
	{
		changed = false;
	}
	
	var hasSubmitContent = CtrlUtils.equalsTrue(ctrl.getAttribute("hasSubmitContent"));
		
	if (changed || hasSubmitContent)
	{
		return true;
	}
	
	return false;
};


CtrlUtils.extendedTextIsModifiedByUser = function(attributeId)
{
	var ctrl = dojo.byId(attributeId);
	var isEditMode = ctrl.getAttribute("editMode");
	var origValue = ctrl.getAttribute("origValue");
	var currentValue;
	
	if (CtrlUtils.equalsTrue(isEditMode))
	{
		currentValue = CtrlUtils.getExtendedTextValue(attributeId);
	}
	else
	{
		currentValue = ctrl.getAttribute("currentValue");
	}
	
	var changed = currentValue != origValue;
		
	if ((currentValue == "") && (!origValue))
	{
		changed = false;
	}
	
	return changed;
};


CtrlUtils.getExtendedTextValue = function(attributeId)
{
	var ctrl = dojo.byId(attributeId);
	var isEditMode = ctrl.getAttribute("editMode");
	var retVal;
	
	if (CtrlUtils.equalsTrue(isEditMode))
	{
		// Dijit editor adds a &#160; to the editor when there is no value to be able to place the cursor on FF.
		// On IE it adds it to the end of the string. If you delete everything on FF it returns
		// <br _moz_editor_bogus_node="TRUE" />, so we normalize the string to remove all these.
		retVal = dijit.byId(attributeId).getValue();
		
		if (retVal == '<br _moz_editor_bogus_node="TRUE" />')
		{
			return "";
		}
		
		if (retVal.length > 0)
		{
			//Clear the &#160; at the beginning or end.
			if (retVal.charCodeAt(0) == 160)
			{
				retVal = retVal.substr(1, retVal.length);
			}
			else if (retVal.charCodeAt(retVal.length - 1) == 160)
			{
				retVal = retVal.substr(0, retVal.length - 1);
			}
		}
	}
	else
	{
		retVal = ctrl.getAttribute("currentValue");
	}
	
	return retVal;
};


CtrlUtils.extendedTextHasValue = function (attributeId)
{
	var ctrl = document.getElementById(attributeId);
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return true;
	
	var temp = CtrlUtils.getExtendedTextValue(attributeId);
		
	if (temp == null)
		return false;
		
	temp = temp.replace(/^\s+/, '');
	temp = temp.replace(/\s+$/, '');	
	return ((temp != '') && (temp.length > 0));
};


CtrlUtils.setExtendedTextValue = function(attributeId, value, isModified)
{
	var ctrl = dojo.byId(attributeId);
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return;
		
	ctrl.setAttribute("isModified", isModified);
	ctrl.setAttribute("currentValue", value);
	
	var isEditMode = ctrl.getAttribute("editMode");
	
	if (CtrlUtils.equalsTrue(isEditMode))
	{
		dijit.byId(attributeId).setValue(value);
	}
};

CtrlUtils.extendedTextGrow = function(attributeId, adjustmentFactor)
{
	var editor = dijit.byId(attributeId);
	var height = editor.height;
	height = height.substr(0, height.length - 2);
	height = parseInt(height) + adjustmentFactor;
	height += "px";
	
	var iframe = dojo.byId(attributeId + "_iframe");
	
	if (iframe != null)
	{
		iframe.style.height = height;
	}
	else
	{
		var element = dojo.byId(attributeId);
		
		// FF 3 has child nodes and FF 2 does not.
		if (element.hasChildNodes())
		{
			// The 2nd child is a DIV which has an IFrame that needs to be expanded on FF 3.
			element.childNodes[1].childNodes[0].style.height = height;
		}
		else
		{
			element.style.height = height;
		}
	}
	
	editor.height = height;
};

CtrlUtils.extendedTextShrink = function(attributeId, origHeight, adjustmentFactor)
{
	var editor = dijit.byId(attributeId);
	var height = editor.height;
	height = height.substr(0, height.length - 2);
	
	if ((height - adjustmentFactor) >= origHeight)
	{
		height -= adjustmentFactor;
	}
	else
	{
		height = origHeight;
	}
	
	height += "px";
	
	var iframe = dojo.byId(attributeId + "_iframe");
	
	if (iframe != null)
	{
		iframe.style.height = height;
	}
	else
	{
		var element = dojo.byId(attributeId);
		
		// FF 3 has child nodes and FF 2 does not.
		if (element.hasChildNodes())
		{
			// The 2nd child is a DIV which has an IFrame that needs to be shrunk on FF 3.
			element.childNodes[1].childNodes[0].style.height = height;
		}
		else
		{
			element.style.height = height;
		}
	}
	
	editor.height = height;
};

/************************* Internal Functions *************************/


CtrlUtils._determineOverride = function(override)
{
	// Check if this override is on an object;
	var index = override.indexOf(".");
	
	// Just a normal function if no "." is present.
	if (index == -1)
		return window[override];
	
	// A function that is on an object, like CtrlUtils.something.
	var object = override.substring(0, index);
	var method = override.substring(index + 1, override.length);
	
	return window[object][method];
};


CtrlUtils._getListboxIndexOf = function(ctrl, value)
{
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return 0;
	
	if (ctrl.nodeName != "SELECT")
		return -1;
		
	for (var i = 0; i < ctrl.length; i++)
	{
		if (ctrl.options[i].text == value)
		{
			return i;
		}
	}
	
	return -1;
};


CtrlUtils._getValueListboxIndexOf  = function(ctrl, value)
{
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return 0;
	
	if (ctrl.nodeName != "SELECT")
		return -1;
		
	for (var i = 0; i < ctrl.length; i++)
	{
		if (ctrl.options[i].value == value)
		{
			return i;
		}
	}
	
	return -1;
};


CtrlUtils._getListboxLastIndex = function(ctrl)
{
	var isReadOnly = ctrl.getAttribute("isReadOnly");

	if (CtrlUtils.equalsTrue(isReadOnly))
		return 0;
	
	if (ctrl.nodeName != "SELECT")
		return 0;

	for (var i = ctrl.length - 1; i >= 0; i--)
	{
		if (ctrl.options[i].text != '')
		{
			return i;
		}
	}
	
	return -1;
};


CtrlUtils._markAsChanged = function(ctrl)
{
	var changed = ctrl.getAttribute("currentValue") != ctrl.getAttribute("origValue");
	var hasSubmitContent = CtrlUtils.equalsTrue(ctrl.getAttribute("hasSubmitContent"));
	
	ctrl.setAttribute("isModified", (changed || hasSubmitContent));
	ctrl.setAttribute("isModifiedByUser", changed);
};


CtrlUtils._markTextAsChanged = function(ctrl)
{
	var changed = ctrl.getAttribute("currentValue") != ctrl.getAttribute("origValue");
	
	if ((ctrl.getAttribute("currentValue") == "") && (!ctrl.getAttribute("origValue")))
	{
		changed = false;
	}
	
	var hasSubmitContent = CtrlUtils.equalsTrue(ctrl.getAttribute("hasSubmitContent"));
	
	ctrl.setAttribute("isModified", (changed || hasSubmitContent));
	ctrl.setAttribute("isModifiedByUser", changed);
};


CtrlUtils._isLegacyAttrControl = function(ctrl)
{
	if (ctrl == null || ctrl.getAttribute("version") == null)
	{
		return true;
	}
	
	return false;
};


CtrlUtils._isValidNumber = function(value, errorMessage)
{
	// Test for a valid Number (int or float).				
	if (/^\d*(\.\d+)?$/.test(value))
	{
		return true;
	}
	else
	{
		alert(errorMessage);
		return false;	
	}
};

CtrlUtils._setListboxValue = function(ctrl, value)
{	
	if (!value && value != "") return;
	
	var position = (value == '' ? 0 : CtrlUtils._getListboxIndexOf(ctrl, value));
	
	if (position == -1)
	{
		if (ctrl.getAttribute("formType") != 'SUBMIT')
		{
			position = CtrlUtils._getListboxLastIndex(ctrl) + 1;
			ctrl.options[position] = new Option(value, position);
		}
		else 
		{
			position = 0;
		}
	}
	
	if (ctrl.options.length > 0)
	{
		ctrl.options[position].selected = true;
		
		if (position == 0)
		{
			ctrl.setAttribute("currentValue", "");
		}
		else
		{
			ctrl.setAttribute("currentValue", ctrl.options[position].text);
		}
		
		ctrl.depFunction();
	}
};


CtrlUtils._setValueListboxValue = function(ctrl, value)
{
	if (!value && value != "") return;
		
	var position = (value == '' ? 0 : CtrlUtils._getValueListboxIndexOf(ctrl, value));
	
	if (position == -1)
	{
		if (ctrl.getAttribute("formType") != 'SUBMIT')
		{
			position = CtrlUtils._getListboxLastIndex(ctrl) + 1;
			ctrl.options[position] = new Option(value, value);
		}
		else 
		{
			position = 0;
		}
	}
	
	if (ctrl.options.length > 0)
	{
		ctrl.options[position].selected = true;
		if (position == 0)
		{
			ctrl.setAttribute("currentValue", "");
		}
		else
		{
			ctrl.setAttribute("currentValue", ctrl.options[position].value);
		}
	}
};


/**
 * Returns the index of a item in an array
 * Firefox Array supports indexOf, IE does not.
 */
CtrlUtils.indexOfItemInArray = function(array, item)
{
	for (var i = 0; i < array.length; i++)
	{
		if (array[i] == item)
		{
			return i;
		}
	}
	
	return -1;
};


/**
 * To output the status HTML table.
 */
CtrlUtils.drawVisualStatusTable = function(args)
{
	// Get definition for current lifecycle, prepare primary state list and map for labels
	var lifecycleDefinition = CrProcessUtils.findLifecycleDefinitionForState(args.currentState, args.lifeCycleTransStateInfo);
	
	if (!lifecycleDefinition)
	{
		return;
	}
	
	var primaryStates = CrProcessUtils.createPrimaryStatesArray(lifecycleDefinition);
	var stateNamesToLabelsMap = CrProcessUtils.mapStateNamesToLabels(lifecycleDefinition);
	
	// determine if the current CR is in the sunny day process.  In other words, it is a primary state
	// If it is a primary state, note how far trough the lifecycle it is.

	var sunnyDay = false;
	var statePosition = -1;

	for (var i in primaryStates)
	{
		if (args.currentState == primaryStates[i])
		{
			sunnyDay = true;
			statePosition = i;
		}
	}
	var totalStates = i;

	if (!sunnyDay)
	{
		// state is not in the sunny day process
		
		// identify if there are any outbound trasitions from this state
		var classState = 'in_progress';
		var terminalState = true;

		for (var i in lifecycleDefinition.transitions)
		{
			if (lifecycleDefinition.transitions[i].name.match(args.currentState + '2'))
			{
				terminalState = false;
			}
		}

		// draw the state stand-alone
		window.document.writeln('<TABLE><TR>');
		window.document.writeln('<TD><IMG SRC=' + args.arrowImg + ' ALT=arrow></TD>');
			window.document.writeln('<TD class = ' + classState + '><div>'+stateNamesToLabelsMap[args.currentState]+'</DIV></TD>');
		
		if (!terminalState) 
		{
			window.document.writeln('<TD><IMG SRC=' + args.arrowImg + ' ALT=arrow></TD>');
		}
		
		window.document.writeln('</TR></TABLE>');
	}
	else
	{
		// state is in the sunny day process

		// By default, we have up to statesToShow spaces on the form for displaying states.  
		// If we have more than statesToShow states, display a portion of the lifecycle.

		var startPosition = 0;
		var endPosition = totalStates;

		if (args.statesToShow <= (totalStates))
		{
			// need to adjust the start and finish points of the displayed states

			var displayMidPoint = Math.ceil(args.statesToShow / 2);

			// arrays start at zero so knock one off the mid point
			displayMidPoint--;

			if (statePosition > displayMidPoint)
			{
				startPosition = statePosition - displayMidPoint;
			}
			if (statePosition > (totalStates - displayMidPoint))
			{
				startPosition = totalStates - args.statesToShow + 1;
			}
			endPosition = startPosition + args.statesToShow - 1;

			// don't go beyond total number of states
			if (endPosition > totalStates)
			{
				startPosition--;
				endPosition--;
			}
		}

		window.document.writeln('<TABLE><TR>');

		if (startPosition > 0)
		{
			window.document.writeln('<TD><IMG SRC=' + args.arrowImg + ' ALT=arrow></TD>');
		}

		for (var i = startPosition; i<= endPosition; i++)
		{
			var classState = 'not_done';
			
			if (i < statePosition)
			{
				classState = 'done';
			}
			
			if (i == statePosition)
			{
				classState = 'in_progress';
			}
			
			window.document.writeln('<TD class = ' + classState + '><div>'+stateNamesToLabelsMap[primaryStates[i]]+'</DIV></TD>');

			if (i < totalStates)
			{
				window.document.writeln('<TD><IMG SRC=' + args.arrowImg + ' ALT=arrow></TD>');
			}
		}
		
		window.document.writeln('</TR></TABLE>');
	}
};
