// oslc.js

(function()
{
	// If we're in a subframe, the protocol is in the #fragment of a frame
	// above that includes /rest in the URL.
	// Can't do window.top since that window is probably in an iframe.
	var restWindow = (function() {
		var p = window;

		while (p)
		{
			if (p.location.toString().match(/\/oslc\//))
			{
				return p;
			}

			if (p === p.parent) return;
			p = p.parent;
		}
	})();

	// Public functions are on the 'oslc' object.
	window.oslc = {
		
		formatLabel: function(crid, optionalSynopsis)
		{
			// XXX: We use this weird format since it's what RQM expects. It
			// parses for "'\] \d+:.*". This must match what RTC does...
			var number = /(\d+)$/.exec(crid)[1];
			var label = '[' + crid + '] ' + number + ': ' + 
				(optionalSynopsis ? optionalSynopsis : crid);
			return label;
		},

		/**
		 * Send a CR "create" message to the OSLC container window.
		 * Returns the new CR URL in case their is no parent.
		 * @param cr CRID of new CR.
 		 */
		sendCreate: function(cr, optionalSynopsis)
		{
			if (!restWindow) return;
			url = urlForCr(cr);
			var message =
				'{ "oslc_cm:message": "oslc_cm.create", "oslc_cm:results": ' +
				'[ { "oslc_cm:label": "' + this.formatLabel(cr, optionalSynopsis) + '", "rdf:resource": "' + url + '" } ] }';
			respond(message);
			return url;
		},

		/**
		 * Send a CR "select" message to the OSLC container window.
		 * Returns the new CR URL in case their is no parent.
		 * @param cr CRID of selected CR.
 		 */
		sendSelect: function(cr, optionalSynopsis)
		{
			if (!restWindow) return;
			url = urlForCr(cr);
			var message =
				'{ "oslc_cm:message": "oslc_cm.select", "oslc_cm:results": ' +
				'[ { "oslc_cm:label": "'+ this.formatLabel(cr, optionalSynopsis) + '", "rdf:resource": "' + url + '" } ] }';
			respond(message);
			return url;
		}
	 };
	
	// Private functions

	function urlForCr(cr)
	{
		// We don't really know the right URL for CR, but
		// we'll assume that we're from either a 'select' or 'create' dialog
		// REST URL. We swap the /dialog/foo resource with our /cr/bar resource.
		var dialogUrl = restWindow.location.toString().split("#")[0];

		if (dialogUrl.match(new RegExp('(.+/)dialog/')))
		{
			return RegExp.$1 + "cr/" + encodeURIComponent(cr);
		}
		else
		{
			// :(
			return cr; 
		}
	}

	var PROTOCOL_WINDOWNAME = '#oslc-windowName-1.0';
	var PROTOCOL_POSTMSG = '#oslc-postMessage-1.0';
	var HEADER = "oslc-response:";
	
	function respond(message)
	{
		var protocol = restWindow.location.hash;

		if(protocol == PROTOCOL_WINDOWNAME)
		{
			respondWithWindowName(message);
		}
		else if (protocol == PROTOCOL_POSTMSG)
		{
			try
			{
				respondWithPostMessage(message);

			}
			catch (e)
			{
				// Should never get here is called by a compliant OSLC client.
				alert('Warning: window.postMessage is not supported in this browser. Trying to respond with window.name')
				respondWithWindowName(message);
			}
		}
			
		// Unknown protocol or it was just not specificed. Ignore; might not be an OSLC client calling us.
	}

	function respondWithPostMessage(response)
	{
		restWindow.parent.postMessage(HEADER + response, "*");
	}

	function respondWithWindowName(response)
	{
		var returnUrl = restWindow.name; 
		restWindow.name = response;
		restWindow.location = returnUrl;
	}
})();
