# <trigger>
#       <usage>perl transition_notice.pl notification_templates</usage>
#       <version>4.4</version>
#       <description>This is a template driven notification package commonly used for Email notices in IBM Rational Change.
#                    Templates can be defined either in a ./wsconfig/trigger/templates directory  or in a custom package
#                    that installs the templates into the ./wsconfig/templates/pt/forms/package_name directory.
#                    
#                    A starting point for your own package is provided in the notification_templates package.
#       </description>
#       <argument1>
#               <name>template_package_name</name>
#               <required>false</required>
#               <type></type>
#               <description>The Name of the CS Package that contains the templates.  Note the package must be installed
#                            via the package manager before the templates will be used for notification purposes.
#               </description>
#       </argument1>
# </trigger>

package main;

# Includes 

## CS API Here
use ChangeSynergy::TriggerParser;
use ChangeSynergy::csapi;
use ChangeSynergy::BasicTemplate;
use CGI;
use ChangeSynergy::DefaultHandler;
use ChangeSynergy::DefaultValidation;

## This is the global variable that holds the current delivery handler module.  
my $deliverHandler="UNSET";
my $validationHandler="UNSET";

$template_dir = "./templates";

# WARNING WARNING WARNING WARNING
# This is a bit of test infrastructure that needs to be setup so that we can use a consistant
# userid and setup etc for testing purposes.  If the Env Varialbe CSNOTICE_TEST is set 
# the script runs using ccm_root/ccm_root.
# Variable that needs to be off like this for production
my $testing;  
if ($ENV{'CSNOTICE_TEST'}) {
	$testing = 1;
}

if(@ARGV > 1) {

      
		my $template_arg = shift;
        $template_dir = '../templates/pt/forms/' . $template_arg;
}

#Parse Trigger

# getTriggerDetails

## Parse the trigger file
my $trigger_file = shift;
$trigger = new ChangeSynergy::TriggerParser($trigger_file);


## Get the relevant data out of the Trigger Parser
my $host = $trigger->get_host();
my $port = $trigger->get_port();
my $db   = $trigger->get_database();
my $role = $trigger->get_role();
my $protocol = $trigger->get_protocol();
my $object_id = $trigger->get_object_id();
my $object_type = $trigger->get_object_type();
my %attributes = $trigger->get_attributes();
my %requested_attributes;
my $smtp_server = $trigger->get_smtp_server() || '';
my $from_email_addr = $trigger->get_from_email_addr() || '';
my $dcm_delim = $trigger->get_dcm_delimiter();
my $base_url = $trigger->get_base_url();

# Note this function will only  be available in 4.3 sp2 ... hence the special handling
eval {%requested_attributes = $trigger->get_requested_attributes()};


#Load Data for Template

## Get some stuff about the CR
my $crID = $trigger->get_object_id();
my $from_state = $trigger->get_from_state();
my $to_state = $trigger->get_to_state();
my $user = $trigger->get_user();

# load the email info
$dict{'smtp_server'} = $smtp_server;
$dict{'from_email_addr'} = $from_email_addr;

# load template data
## Load some prelim data
$dict{'crID'} = $crID;
$dict{'from_state'} = $from_state;
$dict{'to_state'} = $to_state;
$dict{'user'} = $user;
$dict{'getDatabase'} = \&getDatabase;

## Trigger data loaded into template structure.
$dict{'host'} = $host;
$dict{'port'} = $port;
$dict{'db'} = $db;
$dict{'role'} = $role;
$dict{'protocol'} = $protocol;
$dict{'object_id'} = $object_id;
$dict{'object_type'} = $object_type;

## Load the User preference functions
$dict{'getEmail'} = \&getEmailAddress;
$dict{'getFullName'} = \&getFullName;
$dict{'getFirstName'}=\&getFirstName;
$dict{'getLastName'}=\&getLastName;
$dict{'getAddress'}=\&getAddress;
$dict{'getCompany'}=\&getCompany;
$dict{'getFax'}=\&getFax;
$dict{'getPhone'}=\&getPhone;
$dict{'getUserPreference'}=\&getUserPreference;
$dict{'getUserAndEmailLink'}=\&getUserAndEmailLink;

## Load the CM Role query functions
$dict{'getUsersWithAllRoles'} = \&getUsersWithAllRoles;
$dict{'getUsersWithAllRolesString'} = \&getUsersWithAllRolesString;
$dict{'getUsersWithAnyRoles'} = \&getUsersWithAnyRoles;
$dict{'getUsersWithAnyRolesString'} = \&getUsersWithAnyRolesString;
$dict{'getUserEmailsWithAllRoles'} = \&getUserEmailsWithAllRoles;
$dict{'getUserEmailsWithAllRolesString'} = \&getUserEmailsWithAllRolesString;
$dict{'getUserEmailsWithAnyRoles'} = \&getUserEmailsWithAnyRoles;
$dict{'getUserEmailsWithAnyRolesString'} = \&getUserEmailsWithAnyRolesString;

$dict{'getEmails'} = \&getEmailAddresses;
$dict{'getEmailAddressesString'} = \&getEmailAddressesString;

$dict{'getLabel'} = \&getLabel;

## Load the Url Stuff
$dict{'getUrl'} = \&getUrl;
$dict{'toHtml'} = \&toHtml;

## Load the report Stuff
$dict{'getReportLink'} = \&getReportLink;
$dict{'doNotDeliver'} = \&doNotDeliver;

## Functions to update the CR with a note in the log
$dict{'addNoteToLog'} = \&addNoteToLog;

## load the attributes that we got from the trigger
## this can be optimized to be a hash to hash operation, but I need to be sure for now that this works.

# Run this over the requested attributes first (if this is a pre transition it will have the value before the change ...)
for $name (keys (%requested_attributes ) ) {

	$dict{$name} = $requested_attributes{$name};
	
}

# Then over the attributes that actually changed ...
for $name (keys (%attributes) ) {

	$dict{$name} = $attributes{$name};
	
}


# Load the plugin register methods
$dict{'registerDeliveryHandler'} = \&registerDeliveryHandler;
$dict{'registerContentPlugin'} =  \&registerContentPlugin; 
$dict{'registerValidationPlugin'} =  \&registerValidationPlugin; 

# Load in the Method to retrieve and cache CS Data
$dict{'loadCRData'} = \&loadCRData;
$dict{'getDate'} = \&getDate;

# Setup CS Connection once
$csapi = new ChangeSynergy::csapi();

$csapi->setUpConnection($base_url);

# create a user object  - Password is not required ... we are using the current user's token

# If the script is running in test mode it will do the login using the data here otherwise
# it uses the login & token info from the trigger file

if ($testing)
{
    print "WARNING: Running log in using Test DB Setup!!\n";
    $csuser = $csapi->Login('ccm_root','ccm_root', 'User','/ccmdb/test' ) || die(" Can't login\n");
}
else 
{
    $csuser = new ChangeSynergy::apiUser($trigger->get_user(), "password", $trigger->get_role(), $trigger->get_token(), $trigger->get_database());
}

# Load all the attribute Labels into the dict
my $all_attrs = $csapi->GetAttributes($csuser);

my $i;
my $j = $all_attrs->getDataSize();

for($i = 0; $i < $j; $i++)
{
	my $key = $all_attrs->getDataObject($i)->getName() . " Label";
	$dict{$key} = $all_attrs->getDataObject($i)->getLabel();
}

# Grab the current user's fullname now that we have a connection
my $fullname = getFullName($user);
$dict{'user_fullname'} = $fullname;

my $template_pattern;

if ($from_state)
{
        $template_pattern = "$template_dir/$from_state" . '2' . "$to_state";
}
else
{
		# Load the crstatus into the dict
		loadCRData('crstatus', \%dict);
		#my $cr = $csapi->GetCRData($csuser, $object_id, "crstatus");
		#$dict{'crstatus'} = $cr->getDataObjectByName("crstatus")->getValue();

        ## Must be an attribute modify trigger!
        my $currentState = $dict{'crstatus'};

        ## get the attribute that caused the trigger ....
        my $attribute_name = "";
        my $attribute_value = "";

        ## There will only be one in here ...
        for $name (keys (%attributes))
		{
                $attribute_value = $attributes{$name};
                $attribute_name = $name;
        }

        ## Build the template match name ...
        $template_pattern = "$template_dir/($attribute_name\.$currentState)|($attribute_name\.all)";

        $dict{'attribute_name'} = $attribute_name;
        $dict{'attribute_value'} = $attribute_value;
}

for $template_name (<$template_dir/*.tmpl>) 
{
        if ($template_name =~ /$template_pattern.*/)
		{
                &handleTemplate($template_name);
        }
}

# getUserPreference
sub getUserPreference 
{
	my $userID = shift;
	my $pref = shift;
	my $item = "unknown";

	eval
	{
		# my $csapi = new ChangeSynergy::csapi();
		# $csapi->setUpConnection($trigger->get_base_url());
		# my $csuser = new ChangeSynergy::apiUser($trigger->get_user(), "password", $trigger->get_role(),
		#$trigger->get_token(), $trigger->get_database());

		## Get the user Preference for the field ... 
		my $tmp = $csapi->GetUserPreference($csuser, $userID, $pref);
		$item = $tmp->getResponseData();	
	};

	if ($@)
	{
		print $@;
	}

	return $item;
}

sub loadCRData
{
	## If we already have the value for the attribute loaded from the trigger file, 
    ## don't retrieve it.  This makes it as fast as possible.

	my $getAttrsRequired = 0;

	my $attribute_list = shift;
	my $dict = shift;   ## Magic reference to $_bt_dict which updates the dict used by the template being processed.
	
	my @attributeList = split(/\|/, $attribute_list);
	
	my $name;
	
	my $new_attribute_list = '';
	my @newAttributes = ();

	for $name (@attributeList)
	{
		unless ($$dict{$name})
		{
			$new_attribute_list = $new_attribute_list . $name . "|";
			@newAttributes = (@newAttributes, $name);
			$getAttrsRequired = 1;
		}
	}
	
	if ($getAttrsRequired)
	{ 
		## Drop the last | sign
		chop $new_attribute_list;
        
        my $cr;
        
        eval 
		{
		    $cr = $csapi->GetCRData($csuser, $object_id, $new_attribute_list);
        };
        
        if ($cr)
		{
            for $name (@newAttributes)
			{
			    $$dict{$name} = $cr->getDataObjectByName($name)->getValue();
			}
        } 
		else
		{
            print "ERROR Retrieving attributes ... verify attributes exist ...\n";
            print "ATTRIBUTE_LIST: $new_attribute_list\n";
        
			for $name (@newAttributes)
			{
			    $$dict{$name} = "unknown: template error loadCRData failed(Bad Attribute)";
			}
        }
    }
}

# Converts a string with mutiple lines into HTML that will keep the line formatting
sub toHtml
{
	my $s = shift;
	$s =~ s/\n/<br>\n/gi;
	return $s;
}


sub sType
{
  my $t = shift;
  return 3 if lc($t) eq "system";
  return 2 if lc($t) eq "shared";
  return 0;
}

sub makeQuery
{
	my $query = shift;
	my $arg;
    my $argNum = 1;
	
	foreach $arg (@_)
	{
         $query =~ s/ARG$argNum/$arg/g;
         $argNum++;
	}
	
	return ($query);
}

# reportName - The name of the report as it is displayed in IBM Rational Change
# reportType - Specifies the type of report. Either SYSTEM or SHARED
# query      - The query to use in the report
# queryType  - The type of query specified. Either SYSTEM, SHARED, or CUSTOM

sub getReportLink
{
	my $reportName = shift;
	my $reportType = shift;
	my $query      = shift;
 	my $queryType  = shift;

	my $url = $base_url;
	$url = $url .  "/servlet/com.continuus.webpt.servlet.PTweb?ACTION_FLAG=admin_form&TEMPLATE_FLAG=BuildingReport&SUPP_ACTION_FLAG=query";
	$url = $url . "&database=" . CGI::escape($db);
	$url = $url . "&role=". $role;

	my $rt = &sType($reportType);

	if ($rt == 0)
	{
  		print "getReportLink:Invalid report type specified: $reportType\nMust be either SYSTEM or SHARED\n";
		return "";
    }
     
	my $qt = &sType($queryType);
	my $n = CGI::escape($reportName);
	$url .= "&SUPP_CHOSEN_REPORT=$n&SUPP_REPORT_CONFIG_TYPE=$rt";

	if ($qt == 0)
	{
        # A custom query was specified instead of a stored query
        my $arg;
        my $argNum = 1;

		foreach $arg (@_)
		{
            $query =~ s/ARG$argNum/$arg/g;
            $argNum++;
        }

     	my $q = CGI::escape($query);
        $url .= "&SUPP_QUERY_STRING=$q";
     }
	 else
	 {
		my $q = CGI::escape($query);
        $url .= "&SUPP_CHOSEN_QUERY=$q&SUPP_QUERY_CONFIG_TYPE=$qt";
	 }

	return $url;
}


sub getDate
{

	my $attribute = shift;
	my $dict = shift;   ## Magic reference to $_bt_dict which updates the dict used by the template being processed.
	
    my $cr;
        
    eval
	{
        $cr = $csapi->GetCRData($csuser, $object_id, $attribute);
    };
        
    if ($cr)
	{
        $$dict{$attribute} = $cr->getDataObjectByName($attribute)->getDate();
    }
	else
	{
        print "ERROR Retrieving attribute $attribute... verify attribute exist ...\n";
    }
}

sub getLabel
{
	my $attr_name = shift;

	return $dict{"$attr_name Label"};
}


sub getDatabase
{
	my $db = $trigger->get_database();
	return $db;
}

sub getUrl
{
	
	my $url = $base_url;
	$url = $url . "/servlet/com.continuus.webpt.servlet.PTweb?ACTION_FLAG=frameset_form&TEMPLATE_FLAG=ProblemReportView";
	$url = $url . "&database=" . CGI::escape($db);
	my $newID = CGI::escape($crID);
	$url = $url . "&role=". $role . "&problem_number=$newID";
	
	return $url;
}

sub getFullName
{
	my $userid = shift;

	return &getUserPreference($userid, "user_name")
}

sub getEmailAddress
{
	my $userid = shift;
	# my $userid = $dict{$user};

	return &getUserPreference($userid, "user_email")
}

sub getEmailAddresses
{
	my @userids = @_;
	my @addresses = ( );
	
	foreach my $userid (@userids)
	{
		my $address = &getEmailAddress ($userid);
	
		if ($address ne "unknown")
		{
			push @addresses, $address;
		}
	}
		
	return @addresses;
}

sub getEmailAddressesString
{
	my $stringDelimiter = shift;
	my @userids = @_;
	
	my @addresses = &getEmailAddresses(@userids);
		
	my $email_list = join $stringDelimiter, @addresses;
	
	return $email_list;
}

sub getFirstName
{
	my $userid = shift;

	return &getUserPreference($userid, "user_first_name")
}


sub getLastName {

	my $userid = shift;
	return &getUserPreference($userid, "user_last_name")

}

sub getAddress
{
	my $userid = shift;

	return &getUserPreference($userid, "user_address")
}

sub getCompany
{
	my $userid = shift;

	return &getUserPreference($userid, "user_submitter_company")
}

sub getFax
{
	my $userid = shift;

	return &getUserPreference($userid, "user_fax")
}


sub getPhone
{
	my $userid = shift;

	return &getUserPreference($userid, "user_phone")
}

## Implement the Role Lookup functions

## Query for the user list from the base model object
sub getUserListFromDB
{
	my @users = ( );
	
	eval
	{
		my $tmp = $csapi->GetNewCV($csuser, "base", "model", "base", "1"); 
		my $cvid = $tmp->getResponseData();   
		$cvid =~ s/CV://;
		$cvid =~ s/\s*$//;
		
		my $objVector = $csapi->GetObjectData($csuser, $cvid, "users"); 
		
		my $user_list = $objVector->getDataObjectByName("users")->getValue();

		@users = split /\n/, $user_list;

	};

	if ($@)
	{
		print $@;
	}

	return @users;
} 

# Process the list and query
sub loadTheMatchingUsers
{
	my $grepStr = shift;
	
	# Get the list of users
	my @users = &getUserListFromDB();
	
	# Grep for the given roles
	my @valid_users = grep { eval($grepStr) } @users;
	
	# Remove everything from the line but the user id
	foreach my $user (@valid_users)
	{
		$user =~ s/user (.*) = .*;/$1/;
	}

	return @valid_users;	
}

sub formulateOrGrepString
{
	my @roles = @_;
	my $grepString = "/.* (";
	
	foreach my $role (@roles)
	{
		$grepString .= "$role|";		
	}
	
	chop $grepString;
	$grepString .= ")( |;).*/";
	
	return $grepString;
}

sub formulateAndGrepString 
{
	
	my @roles = @_;
	my $grepString = "";
	
	foreach my $role (@roles)
	{
		$grepString .= "/.* $role( |;).*/ && ";		
	} 
	
	chop $grepString;
	chop $grepString;
	chop $grepString;	
	chop $grepString;	
	
	return $grepString;
}



sub getUsersWithAllRoles
{
	my @roles = (@_);
	my $grepStr = &formulateAndGrepString (@roles);
	my @users = &loadTheMatchingUsers($grepStr);
	
	return @users;		
}

sub getUsersWithAllRolesString
{
	my $stringDelimiter = shift;
	my @roles = (@_);
	my @users = &getUsersWithAllRoles(@roles);
	my $user_list = join $stringDelimiter, @users;
	
	return $user_list;
}

sub getUsersWithAnyRoles
{
	my @roles = (@_);
	my $grepStr = &formulateOrGrepString (@roles);
	my @users = &loadTheMatchingUsers($grepStr);
	
	return @users;
}

sub getUsersWithAnyRolesString
{
	my $stringDelimiter = shift;
	my @roles = (@_);
	my @users = &getUsersWithAnyRoles(@roles);
	my $user_list = join $stringDelimiter, @users;
	
	return $user_list;
}

sub getUserEmailsWithAllRoles
{
	my @roles = (@_);
	my @users = &getUsersWithAllRoles (@roles);
	my @emails = &getEmailAddresses(@users);
	
	return @emails;
}

sub getUserEmailsWithAllRolesString
{
	my $stringDelimiter = shift;
	my @roles = (@_);
	my @users = &getUserEmailsWithAllRoles(@roles);
	my $user_list = join $stringDelimiter, @users;
	
	return $user_list;
}

sub getUserEmailsWithAnyRoles
{
	my @roles = (@_);
	my @users = &getUsersWithAnyRoles (@roles);
	my @emails = &getEmailAddresses(@users);
	
	return @emails;
}

sub getUserEmailsWithAnyRolesString
{
	my $stringDelimiter = shift;
	my @roles = (@_);
	my @users = &getUserEmailsWithAnyRoles(@roles);
	my $user_list = join $stringDelimiter, @users;
	
	return $user_list;
}


#Supplementary Functions
## function to get cs connection details
sub getConnection
{
	return ($csapi, $csuser);
}

sub buildFourPartName
{

    my $crid = shift;
    my $fourPartName;
	
	if ($dcm_delim)
	{
		my ($subsystem, $number) = split /$dcm_delim/, $crid;
        $fourPartName = $subsystem . "/problem/" ."problem" . $number . "/1";
        
    }
	else
	{
		$fourPartName = "probtrac/problem/problem" . $crid . "/1";
    } 
    
	return $fourPartName;
}

##
## Not sure about how this will work from a plugin ... need to access the csapi ... do we need
## a reference passed to it somehow to the csapi object on initialization ... probably.
sub userWantsEmailNotification
{

	my $userid = shift;
    # return (&getUserPreference($userid, "user_email_notification") ne 'true')

	return 0;
}

sub registerContentPlugin
{
	my $plugin = shift;
	my $bt = shift;
	print "Registering Plugin: $plugin\n";
    eval "use $plugin;";
	my $initPlugin = $plugin . '::initPlugin';
    my $Methods =  $plugin . '::Methods';

	if(!defined(&$initPlugin))
	{
		print "Content Plugin $plugin does not define initPlugin...\n";
		print "Plugin $plugin will not be available\n";
        return;
    }

	## Init the plugin with the rest of the stuff from the registr.
	unless(&$initPlugin(\@_))
	{
		print "Plugin $plugin could not be initialized!\n";
		# return;
	}

	if(!defined(&$Methods))
	{
		print "Plugin $plugin does not define Methods...\nPlugin will not be available\n";
        return;
    }

	my @methods = &$Methods;

	## Load the Methods from the plugin ...
    ## We really should do some name space management here ... like prefix the pluginame to the method somehow ...
	for $methodName (@methods)
	{
		my $completeMethod = $plugin . '::' . $methodName;
        my $longMethodName = $plugin . '__' . $methodName;

		$$bt{$methodName} = \&$completeMethod;
        $$bt{$longMethodName} = \&$completeMethod;
	}
}

sub registerDeliveryHandler
{

	my $plugin = shift;
    eval "use $plugin;";
	my $Handler =  $plugin . '::deliverTemplate';
	my $initPlugin = $plugin . '::initPlugin';

	if(!defined(&$Handler))
	{
		print "Delivery Handler $Handler does not define deliverTemplate...\n";
		print "Using Default Handler: $deliverHandler\n";
       	return;
    }

	if(!defined(&$initPlugin))
	{
		print "Delivery Handler $plugin ($initPlugin) does not define initPlugin...\n";
		print "Using Default Handler: $deliverHandler\n";
       	return;
    }

	## Init the plugin with the rest of the stuff from the registr.
	unless(&$initPlugin(@_))
	{
		return;
	}

	$deliverHandler = $Handler;
	return;
}

sub registerValidationPlugin 
{
	my $plugin = shift;
    eval "use $plugin;";

    my $Handler =  $plugin . '::validateData';
	my $initPlugin = $plugin . '::initPlugin';

	if(!defined(&$Handler))
	{
		print "Validation Handler $Handler does not define validateData...\n";
		print "Using Default Handler: $validationHandler\n";

       	return;
    }

	if(!defined(&$initPlugin))
	{
		print "Validation Handler $plugin ($initPlugin) does not define initPlugin...\n";
		print "Using Default Handler: $validationHandler\n";

       	return;
    }

	## Init the plugin with the rest of the stuff from the registr.
	unless(&$initPlugin(@_))
	{
		return;
	}
    
	$validationHandler = $Handler;

	return;
}

$handleTemplate = 1;

sub doNotDeliver
{
  $handleTemplate = 0;
}

sub handleTemplate
{
	$tmpl = shift;
	
	## set the default Handler ... email handler.
	registerDeliveryHandler('ChangeSynergy::DefaultHandler');
    registerValidationPlugin('ChangeSynergy::DefaultValidation');

	$bt = ChangeSynergy::BasicTemplate->new(strip_email_comments => 1, default_undef_identifier => 1);
	# We need this so that we can at least warn about undefined values in the template
    $bt->{default_undef_identifier} = 'Undefined Attribute';
    # This makes it easy to use includes in templates ... make sure the include type is virtual
    $bt->{include_document_root} = $template_dir;
    
	# By default handle the sending of the template if there is a valid
	# handler.  If the template calls doNotDeliver(), then don't handle it.

	$handleTemplate = 1;
	my $template = $bt->parse($tmpl,\%dict, );

    if (&$validationHandler(\%dict) && $handleTemplate)
	{
		&$deliverHandler($template);
	}
	else
	{
        print "INFO: template validation rules prohibit sending expanded template to handler\n";
    }
}

# Function to add a note to the current CR log
# Takes a single parameter which is the note to add
sub addNoteToLog
{
	my $note = shift;
	
	## Get a connection to the CR
	my $cr = $csapi->GetCRData($csuser, $crID, "_COMMENTS");
	
	## Modify the CR to have the current date ...
	$cr->getDataObjectByName("_COMMENTS")->setValue("$note");

	my $result = $csapi->ModifyCR($csuser, $cr);
}

sub getUserAndEmailLink
{
	my $userid = shift;
	my $tempEmail = getEmailAddress($userid);
	my $tempName = getFullName($userid);
	my $result;
	
	if (($tempEmail eq "N/A" || length($tempEmail) == 0) && ($tempName eq "N/A" || length($tempName) == 0))
	{
		$result = $userid;
	}
	elsif ($tempEmail eq "N/A" || length ($tempEmail) == 0)
	{
		$result = $tempName . "($userid)";
	}
	else
	{
	    $result = "<a href='mailto:" . $tempEmail . "'>".
		                   $tempName . "($userid) </a>";
	}
	return $result;
}
